/*
Copyright (C) 2009 -  Claudio Salvadori & Paolo Pagano 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/**
 *\file documentation.h
 *\brief  Documentation pages
 */

/** 
* \mainpage DAQ: a ROOT based Data AcQuisition platform for debugging embedded devices
* \section motivation Motivation
* In Embedded Systems the validation of software solutions
* is quite difficult with respect to high-end devices 
* for the lack of ordinary debugging tools like display,
* file system, etc.
* The communication towards a PC permits to debug the software code making
* use of tools available in high-end devices only (awk, grep, find, etc.)
* Moreover it is quite beneficial to implement data
* serialization in order to assess the performances of the embedded system
* firmware (implementing some kind of logic)
* with respect to full-fledged algorithms available in high end devices only.
* We propose this framework including serialization and post-processing virtues
* coming from the ROOT framework developed at CERN with standardization and
* extendibility feautures coming from the xml language.
* We propose an example to test the performances 
* of a full-customed firmware for onboard image processing.
*
* \section arch Software architecture 
* The architecture of DAQ is based essentially on 5 blocks: 
* - \b xml \b parser to interpret the communication specs (physical specs, handshake feature, data pattern...);
* - \b data \b reader to read raw data directly from the API functions;
* - \b data \b formatter to format data following the given data pattern;
* - \b self \b configuring \b on-line \b data \b renderer to render data following the xml presets;
* - \b ROOT \b serializer to store data into a repository (ROOT file) on the filesystem.
*
* The working principle is based on the feature extraction from xml file and the creation of 2 data structures:
* one to define the reader tasks and one to describe the packet content in terms of fields (vector of metadata). For such puroposes the structs SDef and SMData are defined.
So the logic follows these steps:
* - start of the communication (initialization and handshake)
* - syncronization with the stream (by lenght or by the start and end symbols)
* - raw data reading
* - raw data processing (decoding and fragmentation in messages prior to interpretation)
* - data formatting following the metadata definition
*
* The output of this system is a vector of vector of \b SData, a structure
* that contains data formatted plus the information needed for the case of
* casting and processing.  
* \image latex general.eps "DAQ architecture"width=10cm 
* \image html general.png "DAQ architecture" width=2cm 
* \image latex Reader.eps "Reader structure" width=10cm 
* \image html Reader.png "Reader structure" width=2cm 
* The formatted data can be stored in a structured
* repository called ROOT designed by CERN to store data from  experiments
* in High Energy Physics. ROOT Framework implements procedures for data debugging
* permitting post-processing (off-line) of "structured" data: this
capability is very valuable whenever direct means for system evaluation
are absent asin the case of very simple embedded systems (having no display, no filesystem
reduced I/O functionalities).
*  Evidently to render flexible and adaptive these capabilities
* some coding intervention is necessary especially for the interface between DAQ
* and ROOT (files DAQSData.cxx and DAQPayload.h in the directory rootbulk).   
* \image html interface.png "Interface DAQ-ROOT" width=2cm
* \image latex interface.eps "Interface DAQ-ROOT" width=10cm
* \page usage Usage 
* \section installation Installation 
* DAQ platform is possible to compile under Linux and under Windows POSIX emulator Cygwin. To compile
* correctly DAQ platform is necessary to install:
* - Xerces lib (install directly from packet manager: libxerces-c2-dev and libxerces-c28)
* - FLTK libraries (install directly from packet manager: libfltk1.1 and libfltk1.1-dev). If is used Cygwin see also \ref cygwin "Cygwin issues"
* - JPEG libraries (install directly from packet manager: libjeg62 and libjeg62-dev)
* - ROOT framework (downlodable from http://root.cern.ch/drupal/). If is used Cygwin see also \ref cygwin "Cygwin issues".
*
**\subsection cygwin Cygwin issues 
* To use FLTK libraries on Cygwin it is needed to fix the jmorecfg.h file, replacing:
* - \verbatim #ifndef HAVE_BOOLEAN \endverbatim with \verbatim #if (!defined(HAVE_BOOLEAN)) && (!defined(_RPCNDR_H)) \endverbatim
* - \verbatim #ifndef XMD_H \endverbatim with \verbatim#if (!defined(XMD_H) && !defined(_BASETSD_H)) \endverbatim
*
* The only ROOT version working on Cygwin is the 5.11 version, that's included in this package. 
* \warning Remember that ROOT under Cygwin is unsupported.
*
* We provide the DAQ system for Cygwin as a "unsupported" product;
* We will not guarantee to help you in solving Cygwin/ Windows related bugs/ misbehaviors.
* \section conf System personalization
* \subsection impl How to add a device module?
* In this subsection it is explained how to add a new stream class to interface to another communication device. To do this follow
* these steps:
* - Implement the class related to the communication device inheriting from IStreamDaq class.
* - Update the \ref schema with the name of the new communication device.
* - Update the enumeration of the devices in daq_enum.h  
* - Update the xml parsing procedure
* - Update the error enumeration
* - Update the CReader class
*
* \subsection impl1 How to add a renderer module?
* In this subsection it is explained how to add a new data renderer class. To do this follow these steps:
* - Update the \ref schema with the renderer presets (if they need).
* - Implement the class related to renderer (inheriting from IRenderer class) and the related exception handler class.
* \warning The preset parsing procedure has to be done inside the renderer class
* - Register the renderer class in the object-factory
* \remarks The registering procedure permits to link the renderer name defined in the xml data semantic attribute with the 
* respective renderer class. For example, the following code shows how the image renderer is registered:
* \verbatim IRenderer::Factory registerFactoryI("image", CImage::createObj); \endverbatim
* \see CImage.cpp
*
* \subsection conf How to customize DAQ to acquire and store data?
* In this subsection it is explained how to interface DAQ platform with the embedded system. To do this follow
* these steps:
* - customize the xml file following xml-schema (see \ref schema)
* - customize DAQSData.cxx and DAQPayload.h files to interface DAQ with ROOT repository
*
* \page schema Xml-Schema
* \include input.xsd
*
* \page XML_ How to fill the XML-file?
* \section intro_xml Introduction
* This XML file fixes:
* - the communication device settings, 
* - the handshake word to initialize the communication, 
* - the packet lenght,
* - the data topology inside a packet,
* - the renderer the show data.
* \subsection title_xml_intro Stream title (REQUIRED)
* In this node is defined the title of the current data acquistion session. 
* \subsection stream-xml_intro Stream definition 
* \{
* All the attributes from this node and from his children nodes describe the communication device presets. \n \n
* \b stream node (REQUIRED): \n
* \{
* - stream_type: type of stream (the allowed value are file, serial, pipe, socket). (REQUIRED)
* - stream_name: the name or the address of the file to be written (/dev/ttyUSB0 for serial, or IP-address or 'localhost' for socket, or the name of the file,
* or the pipe name). (REQUIRED) 
* - stream_info: more info about the stream (i.e. the port number for the socket). (REQUIRED only ofo socket)
* \}
* \b serial_stream node (REQUIRED for serial communication):\n
* \{ 
* - baud_rate: baud rate value (allowed values: 9600, 19200, 38400, 57600, 115200) (REQUIRED)
* - data_bits: number of bits for symbol (allowed values: 5, 6, 7, 8)  (REQUIRED)
* - parity: parity bit (allowed values: odd, even, none)  (REQUIRED)
* - stop_bits: numeber of stop bits (allowed values: 1, 1.5, 2)  (REQUIRED)
* \}
* \b file_stream node (REQUIRED for files):\n
* \{ 
* - header_l: file header length (NOT REQUIRED)
* - n_entry_s: number of payload entries start bytes (inside the header) (NOT REQUIRED)
* - n_entry_l: number of payload entries length in bytes (inside the header) (NOT REQUIRED)
* \}
* \b stream_opt node (NOT REQUIRED):\n
* \{
* - decoder: used decoder type (allowable value are 'none' and 'xor') \n
* - esc_char: escape character (for decoder). Could be expressed like a character or like an hexadecimal number (see s_type_opt) \n
* - start_char: syncronization start char. Could be expressed like a character or like an hexadecimal number (see s_type_opt)  \n
* - end_char: syncronization end char. Could be expressed like a character or like an hexadecimal number (see s_type_opt)  \n
* - s_type_opt: the way to interpret the selected character: \n 
* 1. 'char' all the character are interpreted like character; \n
* 2. 'hex' all the character are interpreted like hexadecimal value.\n
* \}
* \}
* \subsection handshake_ Handshake definition (REQUIRED)
* The attribute of the node handshake explain wich type of handshake is choosen:
* \{
* - 'none': no handshake
* - 'start': handshake is done to syncronize DAQ with the stream and/or to set-up the device.
* - 'standard': the packet to be parsed by DAQ is an answer to the unique handshake packet.
* \}
* Every handshake packet is describe by this field: (NOT REQUIRED)
* \{
* - handshake_send: packet to send (REQUIRED in case of 'start' and 'standard') 
* - handshake_ack: acknowledgement to the packet (only in case of 'start') 
* - handshake_nack: not acknowledgement to the packet (only in case of 'start')  
* - h_type_pkt: the way to interpret the selected character: \n 
* 1. 'char' all the character are interpreted like character; \n
* 2. 'hex' all the character are interpreted like hexadecimal value.\n
* \}
* \subsection payload_intro Payload definition  
* All the attribute from this node and from all his children nodes describe the stream payload.
* \b packet_description node (REQUIRED): \n
* \{
* - length_pkt : total packet length in bytes (REQUIRED)
* \}
* \b data node (REQUIRED): \n
* \{
* - data_name: name of the field (REQUIRED) \n
* - data_position: absolute position from the start of the packet in number of bytes (REQUIRED)  \n
* - data_size: size of the field in number of bytes (REQUIRED)  \n
* - data_type: data type (INT8, UINT8, INT16, UINT16, INT32, UINT32, INT64, UINT64, FLOAT32, FLOAT64) (REQUIRED)  \n
* - type: semantic of the data (nothing, table, image, histogram, graph, target) (REQUIRED). Every different renderer needs 
* to fill the related presets nodes.
* - endianess: specify the endianess of the data (big_endian, little_endian) (REQUIRED)
* \}
* \b table_preset (REQUIRED if the data semantic is "table"): \n
* \{
* - table_name: destination file name where data will be stored;
* \}
* \b graph_preset (REQUIRED if the data semantic is "graph"): \n
* \{
* - graph_name: name of the graph;
* - graph_title: title of the graph;
* - graph_max: maximum value for x axis.
* \}
* \b histogram_preset (REQUIRED if the data semantic is "histogram"): \n
* \{
* - histogram_name: name of the histogram;
* - histogram_title: title of the histogram;
* - histogram_nbins: number of bins;
* - histogram_min: minimum value for x axis;
* - histogram_max: maximum value for x axis.
* \}
* \b canvas_preset (REQUIRED to show "graph" or "histogram"): \n
* \{
* - canvas_title: ROOT canvas title;
* - canvas_width: ROOT canvas width;
* - canvas_height: ROOT canvas height.
* \}
* \b image_preset (REQUIRED if the data semantic is "image"): \n 
* \{
* - image_name: image window name;
* - image_width: image width;
* - image_height: image height;
* - image_channel: number of color channel.
* \}
* 
* \warning These five character entities are assumed to be predeclared, and it is possible to use without declaring them:
* - "&lt;": the less-than character (<) starts element markup (the first character of a start-tag or an end-tag).
* - "&amp;": the ampersand character (&) starts entity markup (the first character of a character entity reference).
* - "&gt;": the greater-than character (>) ends a start-tag or an end-tag.
* - "&quot;": the double-quote character (") can be symbolised with this character entity reference when you need to 
* embed a double-quote inside a string which is already double-quoted.
* - "&apos;": the apostrophe or single-quote character (') can be symbolised with this character entity reference when 
* you need to embed a single-quote or apostrophe inside a string which is already single-quoted.
*
* \page socket1 EXAMPLE : acquisition of a stream from a socket (1)
* DAQ libraries give the possibilities to communicate and parse data from a socket, where the DAQ is the client
* and the source of the stream is the server. In this case it is implemented a simple server thread (see SocketExample1.cpp)
* that after a simple handshake sends packets shaped like: <DAQ*NNNNNNNN>, where <, > are respectively the start and 
* the end syncronization character, and NNNNNNNN is an UINT32 counter (this value will be saved in a file called "Counter").
* \section example_s1 How does SocketExample1.cpp work?
* \dontinclude SocketExample1.cpp
* This is the implementation of a socket server to communicate with DAQ; the port number is passed as an argument (i.e. 
* ./exe/SocketExample1 2000). 
* \skip argv
* \until {
* Checking the correct sintax for SocketExample1 execution: port number has to be passed as parameter
* \skip if
* \until }
* Opening a socket to be used in internet domain using TCP
* \skip sockfd
* \until }
* Filling the server address structure
* \skip bzero
* \until number
* Binding operation (server side only)
* \skip sockfd
* \until }
* Listening for clients connection (blocking function)
* \skip sockfd
* \until clilen
* Accepting a client and connection
* \skip newsockfd
* \until }
* Handshake:
* \{
* - Reading the handshaking word
* \skip bzero
* \until }
* - Checking if the handshaking packet is correct (it has to be 1) and answer...
* \skipline strcmp
* ... ACK if the packet is correct, ...
* \skip ACK
* \until }
* ...NAK otherwise
* \skip NAK
* \until }
* \}
* Infinite cycle for upgrading the counter, escaping the char and sending packets
* \skip while
* \until return
*
* \section xml_socket2 XML file example for socket acquisition
* This file.xml fixes the socket settings, the handshake word to initialize the communication, the packet lenght
* and the data topology in a packet.
* \dontinclude socketLinux_1.xml
* \subsection title_s1 Stream title 
* \skipline title
* \subsection stream_s1 Stream definition
* \{
* In this case DAQ will acquire data from a socket:
* - connected to the IP-address 'localhost' through the port number 2000;
* - stream start character: 0x02;
* - stream end character: 0x03;
* - escape character: 0x20 (using xor decoder).
* \}
* \skip stream
* \until /stream
* \subsection_s1 handshake Handshake definition 
* The handshake type is 'start', the packet to send is:
* \{
* - '1' (0x31)
* \}
* ACK packet is:
* \{
* - '1' (0x31)
* \}
and NAK packet is: 
* \{
* - '0' (0x30)
* \}
* \skip handshake
* \until /handshake
* \subsection payload_s1 Payload definition  
* \skip packet_description
* \until /packet_description
* \subsection xml_s1 socketLinux_1.xml
* \include socketLinux_1.xml
*
* \section pay_socket_s1 Payload analysis example
* In this case the DAQEvent class is projected to store data from the socket, and in the main(...) function
* (in DAQ.cpp) the event->Build(...) function is called every packet received.
* \verbatim
 		event->Build(ev++, data_formatted);
		tree->Fill();  //fill the tree
\endverbatim
*
* \section coding_s1 Coding intervention
* To configure DAQ is needed to modify the code in 2 files:
* - DAQPayload.h
* - DAQSData.cxx
* \subsection payload_s1 Payload definition (DAQPayload.h)
* \dontinclude DAQPayload.h
* Definition of buffer size:
* \skip DAQSOCKET1
* \until #endif
* Definition of the payload structure:
* \skip DAQSOCKET1
* \until #endif
* \subsection sdata_s1 DAQSData class (DAQSData.cxx)
* \dontinclude DAQSData.cxx
* Copy constructor:
* \skip DAQSOCKET1
* \until #endif
* Default constructor:
* \skip DAQSOCKET1
* \until #endif
* Constructor:
* \skip DAQSOCKET1
* \until #endif
* Print data method:
* \skip DAQSOCKET1
* \until #endif
*
* \page socket2 EXAMPLE : acquisition of a stream from a socket (2)
* DAQ libraries give the possibilities to communicate and parse data from a socket, where the DAQ is the client
* and the source of the stream is the server. In this case it is implemented a simple server thread (see SocketExample2.cpp)
* that after a simple handshake sends packets shaped like: <DAQ*NNNNNNNN*MMMMMMMM>, where <, > are respectively the start and 
* the end syncronization character, and a couple of UINT32 random value (NNNNNNNN, MMMMMMMM) rendered respectively in a graph
* and in a histogram.
* \section example_s2 How does SocketExample2.cpp work?
* \dontinclude SocketExample2.cpp
* This is the implementation of a socket server to communicate with DAQ; the port number is passed as an argument (i.e. 
* ./exe/SocketExample2 2000). 
* \skip argv
* \until {
* Checking the correct sintax for SocketExample2 execution: port number has to be passed as parameter
* \skip if
* \until }
* Opening a socket to be used in internet domain using TCP
* \skip sockfd
* \until }
* Filling the server address structure
* \skip bzero
* \until number
* Binding operation (server side only)
* \skip sockfd
* \until }
* Listening for clients connection (blocking function)
* \skip sockfd
* \until clilen
* Accepting a client and connection
* \skip newsockfd
* \until }
* Handshake:
* \{
* - Reading the handshaking word
* \skip bzero
* \until }
* - Checking if the handshaking packet is correct (it has to be 1) and answer...
* \skipline strcmp
* ... ACK if the packet is correct, ...
* \skip ACK
* \until }
* ...NAK otherwise
* \skip NAK
* \until }
* \}
* Infinite cycle for upgrading the 2 random value, escaping the char and sending packets
* \skip while
* \until return
*
* \section xml_socket_s2 XML file example for socket acquisition
* This file.xml fixes the socket settings, the handshake word to initialize the communication, the packet lenght
* and the data topology in a packet.
* \dontinclude socketLinux_2.xml
* \subsection title_s2 Stream title 
* \skipline title
* \subsection stream_s2 Stream definition
* \{
* In this case DAQ will acquire data from a socket:
* - connected to the IP-address 'localhost' through the port number 2000;
* - stream start character: 0x02;
* - stream end character: 0x03;
* - escape character: 0x20 (using xor decoder).
* \}
* \skip stream
* \until /stream
* \subsection handshake_s2 Handshake definition 
* The handshake type is 'start', the packet to send is:
* \{
* - '1' (0x31)
* \}
* ACK packet is:
* \{
* - '1' (0x31)
* \}
and NAK packet is: 
* \{
* - '0' (0x30)
* \}
* \skip handshake
* \until /handshake
* \subsection payload_s2 Payload definition  
* \skip packet_description
* \until /packet_description
* \subsection xml_s2 socketLinux_2.xml
* \include socketLinux_2.xml
*
* \section pay_socket_s2 Payload analysis example
* In this case the DAQEvent class is projected to store data from the socket, and in the main(...) function
* (in DAQ.cpp) the event->Build(...) function is called every packet received.
* \verbatim
 		event->Build(ev++, data_formatted);
		tree->Fill();  //fill the tree
\endverbatim
*
* \section coding_s2 Coding intervention
* To configure DAQ is needed to modify the code in 2 files:
* - DAQPayload.h
* - DAQSData.cxx
* \subsection payload_s2 Payload definition (DAQPayload.h)
* \dontinclude DAQPayload.h
* Definition of buffer size:
* \skip DAQSOCKET2
* \until #endif
* Definition of the payload structure:
* \skip DAQSOCKET2
* \until #endif
* \subsection sdata_s2 DAQSData class (DAQSData.cxx)
* \dontinclude DAQSData.cxx
* Copy constructor:
* \skip DAQSOCKET2
* \until #endif
* Default constructor:
* \skip DAQSOCKET2
* \until #endif
* Constructor:
* \skip DAQSOCKET2
* \until #endif
* Print data method:
* \skip DAQSOCKET2
* \until #endif
*
* \page serial EXAMPLE : serial acquisition of a picture from HV7131GP and FLEX
* The firmware is downlodable from http://www.evidence.eu.com/content/view/329/266/.
*
* \section xml_image XML file example for image acquisition via RS-232
* This file.xml fixes the RS-232 settings, the handshake word to initialize the camera, the packet lenght
* and the data topology in a packet.
* \dontinclude serialImageLinux.xml
* \subsection Title_image Stream title 
* \skipline title
* \subsection Stream_image Stream definition
* \{
* The serial stream selected will have baud-rate 115200, 8 bits for character, no parity bit and 1 stop bit.
* \skip baud
* \until baud
* \}
* \subsection Handshake_image Handshake definition
* The handshake type is 'start', the packets to send are:
* \{
* - '1' (0x31)
* - '*01*01*' (0x2A 0x30 0x31 0x2A 0x30 0x31 0x2A)
* \}
* ACK packets are:
* \{
* - '1' (0x31)
* - '1' (0x31)
* \}
and NAK packets are: 
* \{
* - '0' (0x30)
* - '0' (0x30)
* \}
* \skip handshake
* \until /handshake
*
* \subsection payload_image Payload definition
* \skip packet_description
* \until /packet_description
* \subsection Xml_image serialImageLinux.xml
* \include serialImageLinux.xml
*
* \section pay_image Payload analysis example
* In this case the DAQEvent class is projected to store an image of 19200 bytes (160x120 pixel) and the other elements of the packet,
* and in the main(...) function (in DAQ.cpp) the event->Build(...) function is called every packet received.
* \verbatim 
event->Build(ev++, data_formatted);
tree->Fill();  //fill the tree \endverbatim
*
* \section coding_image Coding intervention
* To configure DAQ is needed to modify the code in 2 files:
* - DAQPayload.h
* - DAQSData.cxx
* \subsection payload_image Payload definition (DAQPayload.h)
* \dontinclude DAQPayload.h
* Definition of buffer size:
* \skip DAQIMAGE
* \until #endif
* Definition of the payload structure:
* \skip DAQIMAGE
* \until #endif
* \subsection sdata_image DAQSData class (DAQSData.cxx)
* \dontinclude DAQSData.cxx
* Copy constructor:
* \skip DAQIMAGE
* \until #endif
* Default constructor:
* \skip DAQIMAGE
* \until #endif
* Constructor:
* \skip DAQIMAGE
* \until #endif
* Print data method:
* \skip DAQIMAGE
* \until #endif
*
* \section post_image Post-processing images
* The file LoopDAQEvent_Image.cpp is the application to process the images stored inside ROOT repository ( default name is \b test.root ). The paradigm
* of this simple programm is:
* \dontinclude LoopDAQEvent_Image.cpp
* - read the element
* \skipline getPayload()
* - calculate the likely-hood value using FDY and filling histogram
* \skip fdy_cd
* \until canv_->Update();
* - show stored image
* \skip initRenderer
* \until renderImage
*
* \page sniffer EXAMPLE : acquisition of a stream from a sniffer file
* DAQ libraries give the possibilities read and parse data from a binary files, like the ones pruduced by the sniffer CC2420DK
* by Texas Instruments. In this case it is written an XML file to parse one of the files contained in the test_beacon folder: 
* using the post-processing procedure is possible to display the beacon-coerency histogram for the OpenZB implementation over 
* TinyOS.

* \section xml_sniffer XML file example for sniffer acquisition
* This XML file permit to DAQ to read and parse a file generated by the CC2420DK sniffer.
* \dontinclude file_sniffer.xml
* \subsection title_sniffer Stream title 
* \skipline title
* \subsection stream_sniffer Stream definition
* \{
* First of all is fixed the file-reading presets: the name of the file, the file structure...
* In this case DAQ will acquire data from the file:
* - called test_beacon/BO1SO1.psd;
* - the file header is long 4 bytes;
* - the number of entries are written inside the header at position 0 (length 4 bytes).
* \}
* \skip stream
* \until /stream
* \subsection handshake_sniffer Handshake definition 
* The handshake type is 'none'.
* \skipline handshake
* \subsection payload_sniffer Payload definition  
* \skip packet_description
* \until /packet_description
* \subsection xml_sniffer file_sniffer.xml
* \include file_sniffer.xml
*
* \section pay_sniffer Payload analysis example
* In this case the DAQEvent class is projected to store data from the socket, and in the main(...) function
* (in DAQ.cpp) the event->Build(...) function is called every packet received.
* \verbatim
event->Build(ev++, data_formatted);
tree->Fill();  //fill the tree
\endverbatim
*
* \section coding_sniffer Coding intervention
* To configure DAQ is needed to modify the code in 2 files:
* - DAQPayload.h
* - DAQSData.cxx
* \subsection payload_sniffer Payload definition (DAQPayload.h)
* \dontinclude DAQPayload.h
* Definition of buffer size:
* \skip DAQSNIFFER
* \until #endif
* Definition of the payload structure:
* \skip DAQSNIFFER
* \until #endif
* \subsection sdata_sniffer DAQSData class (DAQSData.cxx)
* \dontinclude DAQSData.cxx
* Copy constructor:
* \skip DAQSNIFFER
* \until #endif
* Default constructor:
* \skip DAQSNIFFER
* \until #endif
* Constructor:
* \skip DAQSNIFFER
* \until #endif
* Print data method:
* \skip DAQSNIFFER
* \until #endif
*/
